@@ -1,8 +1,9 @@
 from typing import Union
 
 import pytest
-from pydantic import BaseModel, Field, RootModel, create_model
+from pydantic import BaseModel, ConfigDict, Field, RootModel, create_model
 
+from json_schema_to_pydantic import PydanticModelBuilder
 from json_schema_to_pydantic.builders import ConstraintBuilder
 from json_schema_to_pydantic.exceptions import CombinerError
 from json_schema_to_pydantic.handlers import CombinerHandler
@@ -19,7 +20,11 @@ def create_handler():
     # Define simple callbacks for testing purposes
     # Note: These might need adjustment if tests require full model building recursion
     def simple_recursive_builder(
-        schema, root_schema, allow_undefined_array_items=False, allow_any_type=False
+        schema,
+        root_schema,
+        allow_undefined_array_items=False,
+        allow_any_type=False,
+        populate_by_name=False,
     ):
         # Resolve $ref first, similar to PydanticModelBuilder._get_field_type
         if "$ref" in schema:
@@ -39,7 +44,11 @@ def create_handler():
                 )
                 for n, p in props.items()
             }
-            return create_model("NestedTestModel", **fields)
+            return create_model(
+                "NestedTestModel",
+                __config__=ConfigDict(populate_by_name=populate_by_name),
+                **fields,
+            )
         elif "oneOf" in schema:
             # Delegate back to a temporary handler instance for nested oneOf
             # This is a bit complex for a simple test setup, might need refinement
@@ -49,9 +58,14 @@ def create_handler():
                 reference_resolver,
                 simple_recursive_builder,
                 simple_field_info_builder,
+                PydanticModelBuilder._sanitize_field_name,
             )
             return temp_handler.handle_one_of(
-                schema, root_schema, allow_undefined_array_items
+                schema["oneOf"],
+                root_schema,
+                allow_undefined_array_items,
+                allow_any_type,
+                populate_by_name,
             )
         elif "anyOf" in schema:
             temp_handler = CombinerHandler(
@@ -62,14 +76,18 @@ def create_handler():
                 simple_field_info_builder,
             )
             return temp_handler.handle_any_of(
-                schema["anyOf"], root_schema, allow_undefined_array_items
+                schema["anyOf"],
+                root_schema,
+                allow_undefined_array_items,
+                allow_any_type,
+                populate_by_name,
             )
         # Fallback to basic type resolver
         return type_resolver.resolve_type(
-            schema, root_schema, allow_undefined_array_items
+            schema, root_schema, allow_undefined_array_items, allow_any_type, populate_by_name
         )
 
-    def simple_field_info_builder(schema, required):
+    def simple_field_info_builder(schema, required, alias=None):
         # Basic field info creation for testing
         kwargs = {}
         constraints = constraint_builder.build_constraints(schema)
@@ -81,6 +99,8 @@ def create_handler():
             kwargs["default"] = None
         if "description" in schema:
             kwargs["description"] = schema["description"]
+        if alias:
+            kwargs["alias"] = alias
         return Field(**kwargs)
 
     # Return the handler instance with dependencies and callbacks
@@ -90,6 +110,7 @@ def create_handler():
         reference_resolver=reference_resolver,
         recursive_field_builder=simple_recursive_builder,
         field_info_builder=simple_field_info_builder,
+        name_sanitizer=PydanticModelBuilder._sanitize_field_name,
     )
 
 
@@ -144,7 +165,7 @@ def test_one_of_handler():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
     assert issubclass(model, BaseModel)
 
 
@@ -214,7 +235,7 @@ def test_one_of_validation():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Test circle
     circle = model(type="circle", radius=5.0)
@@ -242,7 +263,7 @@ def test_empty_combiners():
         handler.handle_any_of([], {})
 
     with pytest.raises(CombinerError):
-        handler.handle_one_of({"oneOf": []}, {})
+        handler.handle_one_of([], {})
 
 
 def test_one_of_invalid_schema():
@@ -297,7 +318,7 @@ def test_one_of_nested():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Test nested discriminated union
     instance = model(type="parent", child={"type": "child1", "value": "test"})
@@ -324,7 +345,7 @@ def test_one_of_required_fields():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Test missing required field
     with pytest.raises(ValueError):
@@ -349,7 +370,7 @@ def test_optional_fields():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Test with only required fields
     instance = model(root={"type": "user", "username": "test"})
@@ -378,7 +399,7 @@ def test_field_descriptions():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Get field info from model
     field_info = model.model_fields["root"].annotation.model_fields
@@ -401,7 +422,7 @@ def test_one_of_invalid_discriminator():
         ]
     }
 
-    model = handler.handle_one_of(schema, {})
+    model = handler.handle_one_of(schema["oneOf"], {})
 
     # Test invalid discriminator value
     with pytest.raises(ValueError):
@@ -463,7 +484,7 @@ def test_one_of_with_ref():
         ]
     }
 
-    model = handler.handle_one_of(schema, root_schema)
+    model = handler.handle_one_of(schema["oneOf"], root_schema)
     assert issubclass(model, RootModel)
 
     # Test dog variant
@@ -548,7 +569,7 @@ def test_one_of_property_with_ref():
 
     # Simulate ModelBuilder call for the property
     prop_schema = root_schema["properties"]["choice"]
-    model = handler.handle_one_of(prop_schema, root_schema)
+    model = handler.handle_one_of(prop_schema["oneOf"], root_schema)
 
     assert issubclass(model, RootModel)
 
@@ -690,3 +711,219 @@ def test_all_of_merging_ref_and_inline():
         model(id=1, description="A long description")  # Missing status
     with pytest.raises(ValueError):
         model(description="A long description", status="active")  # Missing id
+
+
+def test_one_of_simple_type_union():
+    """Test oneOf with simple type variants (Issue #37)."""
+    handler = create_handler()
+
+    schemas = [
+        {"type": "integer"},
+        {"type": "string"},
+    ]
+
+    union_type = handler.handle_one_of(schemas, {})
+    # Should return a Union type, not a discriminated model
+    assert hasattr(union_type, "__args__")
+    assert int in union_type.__args__
+    assert str in union_type.__args__
+
+
+def test_one_of_const_union():
+    """Test oneOf with const values creates a Literal type."""
+    handler = create_handler()
+
+    schemas = [
+        {"const": "red"},
+        {"const": "green"},
+        {"const": "blue"},
+    ]
+
+    literal_type = handler.handle_one_of(schemas, {})
+    # Should return a Literal type
+    from typing import get_args
+
+    assert get_args(literal_type) == ("red", "green", "blue")
+
+
+def test_one_of_const_union_with_falsy_values():
+    """Test oneOf with falsy const values (empty string, 0, null)."""
+    handler = create_handler()
+
+    schemas = [
+        {"const": ""},
+        {"const": 0},
+        {"const": None},
+        {"const": False},
+    ]
+
+    literal_type = handler.handle_one_of(schemas, {})
+    from typing import get_args
+
+    assert get_args(literal_type) == ("", 0, None, False)
+
+
+def test_one_of_ref_without_discriminator():
+    """Test oneOf with $ref variants that don't have a type const discriminator."""
+    handler = create_handler()
+
+    root_schema = {
+        "$defs": {
+            "StringOption": {
+                "type": "object",
+                "properties": {"value": {"type": "string"}},
+            },
+            "IntOption": {
+                "type": "object",
+                "properties": {"value": {"type": "integer"}},
+            },
+        }
+    }
+    schemas = [
+        {"$ref": "#/$defs/StringOption"},
+        {"$ref": "#/$defs/IntOption"},
+    ]
+
+    # Should handle refs without discriminator by falling back to general union
+    union_type = handler.handle_one_of(schemas, root_schema)
+    assert hasattr(union_type, "__args__")
+
+
+def test_one_of_mixed_types():
+    """Test oneOf with a mix of simple types and objects."""
+    handler = create_handler()
+
+    schemas = [
+        {"type": "string"},
+        {"type": "null"},
+        {"type": "object", "properties": {"name": {"type": "string"}}},
+    ]
+
+    union_type = handler.handle_one_of(schemas, {})
+    assert hasattr(union_type, "__args__")
+    # Should include str and None (for null) plus an object model
+    assert str in union_type.__args__
+    assert type(None) in union_type.__args__
+
+
+def test_one_of_const_with_mixed_valid_types():
+    """Test oneOf with const values of different valid Literal types.
+
+    Literal supports str, int, bool, bytes, None.
+    """
+    handler = create_handler()
+
+    schemas = [
+        {"const": "string_value"},
+        {"const": 42},
+        {"const": True},
+        {"const": None},
+    ]
+
+    literal_type = handler.handle_one_of(schemas, {})
+    from typing import get_args
+
+    assert get_args(literal_type) == ("string_value", 42, True, None)
+
+
+def test_one_of_populate_by_name():
+    """Test oneOf handler with populate_by_name=True."""
+    handler = create_handler()
+
+    schema = {
+        "oneOf": [
+            {
+                "type": "object",
+                "properties": {
+                    "type": {"const": "user"},
+                    "username": {"type": "string", "alias": "user_name"},
+                },
+                "required": ["type", "username"],
+            },
+            {
+                "type": "object",
+                "properties": {
+                    "type": {"const": "admin"},
+                    "admin_level": {"type": "integer", "alias": "adminLevel"},
+                },
+                "required": ["type", "admin_level"],
+            },
+        ]
+    }
+
+    model = handler.handle_one_of(
+        schema["oneOf"],
+        {},
+        populate_by_name=True,
+    )
+
+    from typing import get_args
+
+    types = get_args(model.model_fields["root"].annotation)[0].__args__
+    assert len(types) == 2
+    assert types[0].model_config["populate_by_name"] is True
+    assert types[1].model_config["populate_by_name"] is True
+
+
+def test_all_of_populate_by_name():
+    """Test allOf handler with populate_by_name=True."""
+    handler = create_handler()
+
+    schemas = [
+        {
+            "type": "object",
+            "properties": {
+                "first_name": {"type": "string", "alias": "firstName"},
+                "last_name": {"type": "string", "alias": "lastName"},
+            },
+            "required": ["first_name", "last_name"],
+        },
+        {
+            "type": "object",
+            "properties": {
+                "age": {"type": "integer", "alias": "userAge"},
+            },
+            "required": ["age"],
+        },
+    ]
+
+    model = handler.handle_all_of(
+        schemas,
+        {},
+        populate_by_name=True,
+    )
+
+    assert model.model_config["populate_by_name"] is True
+
+
+def test_any_of_populate_by_name():
+    """Test anyOf handler with populate_by_name=True."""
+    handler = create_handler()
+
+    schemas = [
+        {
+            "type": "object",
+            "properties": {
+                "email_address": {"type": "string", "alias": "emailAddress"},
+            },
+        },
+        {
+            "type": "object",
+            "properties": {
+                "phone_number": {"type": "string", "alias": "phoneNumber"},
+            },
+        },
+    ]
+
+    union_type = handler.handle_any_of(
+        schemas,
+        {},
+        populate_by_name=True,
+    )
+
+    from typing import get_args
+
+    types = get_args(union_type)
+    assert len(types) == 2
+    assert types[0].model_config["populate_by_name"] is True
+    assert types[1].model_config["populate_by_name"] is True
